﻿using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.CompilerServices;

namespace System.Collections.Generic {

   public static class CollectionX {

      public delegate IEnumerable GetChilds<T>(T item);

      #region ICollection(mit und ohne T)

      public static T[] ToArray<T>(this System.Collections.ICollection subj) {
         T[] x = new T[subj.Count];
         if (subj.Count > 0) subj.CopyTo(x, 0); return x;
      }

      #endregion//ICollection(mit und ohne T)


      #region Array

      [DebuggerStepThrough()]
      public static T[] Take<T>(this T[] items, int start, int count) {
         start =Math.Max(0, Math.Min(items.Length, start));
         count =Math.Max(0, Math.Min(items.Length - start, count));
         T[] itms = new T[count];
         Array.Copy(items, start, itms, 0, count);
         return itms;
      }

      /// <summary>gibt den **typisierten** Enumerator des Arrays zurück</summary>
      [DebuggerStepThrough()]
      public static IEnumerator<T> GetEnumeratorX<T>(this T[] subj) {
         return ((IList<T>)subj).GetEnumerator();
      }
      [DebuggerStepThrough()]
      public static void CopyX(this Array Arr1, int start1, Array Arr2, int start2, int Length) {
         Array.Copy(Arr1, start1, Arr2, start2, Length);
      }
      [DebuggerStepThrough()]
      public static void CopyX(this Array Arr1, Array Arr2, int start2, int Length) {
         Array.Copy(Arr1, 0, Arr2, start2, Length);
      }
      [DebuggerStepThrough()]
      public static void CopyX(this Array Arr1, Array Arr2, int start2) {
         Array.Copy(Arr1, 0, Arr2, start2, Arr1.Length);
      }
      [DebuggerStepThrough()]
      public static int FindIndex<T>(this T[] Arr, Predicate<T> IsMatch) {
         return Array.FindIndex(Arr, IsMatch);
      }
      [DebuggerStepThrough()]
      public static T Find<T>(this T[] Arr, Predicate<T> IsMatch) {
         return Array.Find(Arr, IsMatch);
      }
      [DebuggerStepThrough()]
      public static int IndexOf<T>(this T[] Arr, T Element) {
         return Array.IndexOf(Arr, Element);
      }
      [DebuggerStepThrough()]
      public static bool Contains<T>(this T[] Arr, T Element) {
         return ((IList<T>)Arr).Contains(Element);
      }
      [DebuggerStepThrough()]
      public static TDest[] ConvertAll<TSrc, TDest>(this TSrc[] Arr, Converter<TSrc, TDest> Converter) {
         return Array.ConvertAll(Arr, Converter);
      }
      [DebuggerStepThrough()]
      public static List<TDest> ConvertAll<TSrc, TDest>(this IEnumerable<TSrc> Arr, Converter<TSrc, TDest> Converter) {
         return (from itm in Arr select Converter(itm)).ToList();
      }
      [DebuggerStepThrough()]
      public static void Sort<T>(this T[] subj) { Array.Sort(subj); }

      #endregion//Array

      #region Dictionary(Of T, T2)

      public static Tvalue GetOrCreate<Tkey, Tvalue>(this Dictionary<Tkey, Tvalue> subj, Tkey key) where Tvalue : new() {
         Tvalue ret = default(Tvalue);
         if (!subj.TryGetValue(key, out ret)) {
            ret = new Tvalue();
            subj.Add(key, ret);
         }
         return ret;
      }
      public static Tvalue GetOrDefault<Tkey, Tvalue>(this Dictionary<Tkey, Tvalue> subj, Tkey key) {
         Tvalue ret = default(Tvalue);
         if (subj.TryGetValue(key, out ret)) return ret;
         return default(Tvalue);
      }
      public static Dictionary<TKey, TItem> ToDictionary<TItem, TKey>(this IEnumerable<TItem> items, Func<TItem, TKey> keySelector) {
         return items.ToDictionary(keySelector, itm => itm);
      }
      public static Dictionary<TKey, TValue> ToDictionary<TItem, TKey, TValue>(this IEnumerable<TItem> items, Func<TItem, TKey> keySelector, Func<TItem, TValue> valueSelector) {
         var result = new Dictionary<TKey, TValue>();
         foreach (var itm in items) result.Add(keySelector(itm), valueSelector(itm));
         return result;
      }

      #endregion//Dictionary(Of T, T2)


      #region IList / IList(Of T)

      [DebuggerStepThrough()]
      public static void Add<T>(this ICollection<T> subj, params T[] itms) {
         foreach (var itm in itms) subj.Add(itm);
      }

      public static int FirstDifference<T>(this IList<T>[] lists, Func<T, object> selector) {
         var commonLength = lists.Min(lst => lst.Count);
         var ubound = lists.Length - 1;
         for (var retVal = 0; retVal < commonLength; retVal++) {
            var cmp = selector(lists[0][retVal]);
            for (var ii = 1; ii <= ubound; ii++) {
               if (!selector(lists[ii][retVal]).Equals(cmp)) return retVal;
            }
         }
         return commonLength;
      }
      public static int FirstDifference(this IList[] lists) {
         var commonLength = lists.Min(lst => lst.Count);
         var ubound = lists.Length - 1;
         for (var retVal = 0; retVal < commonLength; retVal++) {
            var cmp = lists[0][retVal];
            for (var ii = 1; ii <= ubound; ii++) {
               if (!lists[ii][retVal].Equals(cmp)) return retVal;
            }
         }
         return commonLength;
      }
      public static DifferenceResult<T1, T2> DifferenceAnalize<T1, T2>(this IEnumerable<T1> lst1, IEnumerable<T2> lst2, Func<T1, int> conv1 = null, Func<T2, int> conv2 = null) {
         DifferenceResult<T1, T2> retVal = default(DifferenceResult<T1, T2>);
         if (conv1 == null) conv1 = (T1 o) => o.GetHashCode();
         if (conv2 == null) conv2 = (T2 o) => o.GetHashCode();
         var dic1 = new Dictionary<int, T1>();
         retVal.List2_Only = new List<T2>();
         retVal.Mappings = new List<Tuple<T1, T2>>();
         foreach (T1 itm in lst1) dic1.Add(conv1(itm), itm);
         T1 itm1 = default(T1);
         foreach (var itm2 in lst2) {
            var key = conv2(itm2);
            if (dic1.TryGetValue(key, out itm1)) {
               retVal.Mappings.Add(Tuple.Create(itm1, itm2));
               dic1.Remove(key);
            }
            else { retVal.List2_Only.Add(itm2); }
         }
         retVal.List1_Only = dic1.Values.ToList();
         return retVal;
      }
      public struct DifferenceResult<T1, T2> {
         public List<T1> List1_Only;
         public List<T2> List2_Only;
         public List<Tuple<T1, T2>> Mappings;
      }
      public static void InsertOrAdd<T>(this IList<T> subj, int index, T item) {
         subj.Insert(index < 0 ? subj.Count : index, item);
      }
      public static void InsertOrAt0<T>(this IList<T> subj, int index, T item) {
         subj.Insert(index < 0 ? 0 : index, item);
      }
      [DebuggerStepThrough()]
      public static T Clone<T>(this T subj) where T : IList, new() {
         T retVal = new T();
         foreach (var itm in subj) retVal.Add(itm);
         return retVal;
      }

      /// <summary>Bsp: vorletztes Element: list.Peek(-1)</summary>
      [DebuggerStepThrough()]
      public static T Peek<T>(this IList<T> subj, int index = 0) {
         return subj[index + subj.Count - 1];
      }
      /// <summary>Bsp: setze vorletztes Element: list.Poke(-1, "huhu")</summary>
      [DebuggerStepThrough()]
      public static void Poke<T>(this IList<T> subj, T value, int index = 0) {
         subj[index + subj.Count - 1] = value;
      }
      [DebuggerStepThrough()]
      public static T Pop<T>(this IList<T> Subj) {
         var i = Subj.Count - 1;
         var retVal = Subj[i];
         Subj.RemoveAt(i);
         return retVal;
      }
      [DebuggerStepThrough()]
      public static T PopAndPeek<T>(this IList<T> Subj) {
         var i = Subj.Count - 1;
         Subj.RemoveAt(i);
         return Subj[i - 1];
      }
      [DebuggerStepThrough()]
      public static void Swap(this IList Subj, int I1, int I2) {
         var Tmp = Subj[I1];
         Subj[I1] = Subj[I2];
         Subj[I2] = Tmp;
      }
      [DebuggerStepThrough()]
      public static void Swap<T>(this IList<T> Subj, int I1, int I2) {
         var Tmp = Subj[I1];
         Subj[I1] = Subj[I2];
         Subj[I2] = Tmp;
      }
      [DebuggerStepThrough()]
      public static void Swap<T>(this T[] Subj, int I1, int I2) {
         var Tmp = Subj[I1];
         Subj[I1] = Subj[I2];
         Subj[I2] = Tmp;
      }

      /// <summary>Refill oldValues with minimum occurances of CollectionChanged-Event</summary>
      /// <param name="selectors">each selector extracts a significant Property from Item, to decide, whether two items are equal. multiple selectors consider multiple Properties</param>
      public static void ReFillBy<T>(this IList<T> oldValues, IEnumerable<T> newValues, params Func<T, object>[] selectors) {
         using (var ernt = newValues.GetEnumerator()) {
            for (var i = 0; i < oldValues.Count; i++) {
               if (!ernt.MoveNext()) {
                  for (var ii = oldValues.Count; ii-- > i; ) oldValues.RemoveAt(ii);
                  return;
               }
               var itm0 = oldValues[i];
               var itm1 = ernt.Current;
               if (!(object.Equals(itm0, itm1) || (selectors.Length > 0 && selectors.All(sel => object.Equals(sel(itm0), sel(itm1)))))) oldValues[i] = itm1;
            }
            while (ernt.MoveNext()) oldValues.Add(ernt.Current);
         }
      }
      /// <summary> remove all after index, then add newValues </summary>
      public static void AddRangeAt<T>(this IList<T> Subj, IEnumerable<T> newValues, int index = int.MaxValue) {
         using (var ernt = newValues.GetEnumerator()) {
            for (var i = index; i < Subj.Count; i++) {
               if (!ernt.MoveNext()) {
                  for (var ii = Subj.Count; ii-- > i; ) Subj.RemoveAt(ii);
                  return;
               }
               Subj[i] = ernt.Current;
            }
            while (ernt.MoveNext()) Subj.Add(ernt.Current);
         }
      }
      public static void RemoveRangeX(this IList Subj, int start, int count = int.MaxValue) {
         count = Math.Min(count, Subj.Count - start);
         for (var i = start + count; i < Subj.Count; i++) Subj[i - count] = Subj[i];
         var target = Subj.Count - count;
         for (var i = Subj.Count; i-- > target; ) Subj.RemoveAt(i);
      }


      #endregion //IList / IList(Of T)"
            
      #region IEnumerable
      public static List<TResult> Zip<T1, T2, TResult>(this IEnumerable<T1> lst1, Func<T1, T2, TResult> selector, IEnumerable<T2> lst2) {
         var retVal = new List<TResult>();
         using (var e1 = lst1.GetEnumerator()) using (var e2 = lst2.GetEnumerator()) {
            while (e1.MoveNext() && e2.MoveNext()) retVal.Add(selector(e1.Current, e2.Current));
         }
         return retVal;
      }
      public static List<TResult> Zip<T1, T2, T3, TResult>(this IEnumerable<T1> lst1, IEnumerable<T2> lst2, IEnumerable<T3> lst3, Func<T1, T2, T3, TResult> selector) {
         var retVal = new List<TResult>();
         using (var e1 = lst1.GetEnumerator()) using (var e2 = lst2.GetEnumerator()) using (var e3 = lst3.GetEnumerator()) {
            while (e1.MoveNext() && e2.MoveNext() && e3.MoveNext()) retVal.Add(selector(e1.Current, e2.Current, e3.Current));
         }
         return retVal;
      }

      [DebuggerStepThrough()]
      public static IEnumerable<T> Take<T>(this IEnumerable<T> items, int start, int count) {
         return items.Skip(start).Take(count);
      }

      public static IEnumerable<T> SelectMany<T>(this IEnumerable<IEnumerable<T>> outerList) {
         return outerList.SelectMany(innerlist => innerlist);
      }
#if true
      public struct MinMax<T, T2, T3>
         where T2 : IComparable
         where T:T3 {
         public T Min, Max;
         public T2 MinVal, MaxVal;
         private Func<T3,T2> _Selector;
         public readonly bool HasValue;
         public MinMax(T first, Func<T3, T2> selector) {
            _Selector = selector;
            this.Min = first;
            this.Max = first;
            MinVal = _Selector(first);
            MaxVal = MinVal;
            HasValue = true;
         }
         public void Check(T itm) {
            var val = _Selector(itm);
            if (val.CompareTo(MinVal) < 0) {
               Min = itm;
               MinVal = val;
            }
            else if (val.CompareTo(MaxVal) > 0) {
               Max = itm;
               MaxVal = val;
            }
         }
      }
      public static MinMax<T, T2, T3> Extremum<T, T2, T3>(this IEnumerable<T> items, Func<T3, T2> selector)
         where T2 : IComparable
         where T : T3 {
         var enrt = items.GetEnumerator();
         if (!enrt.MoveNext()) { enrt.Dispose(); return default(MinMax<T, T2, T3>); }
         var retVal = new MinMax<T, T2, T3>(enrt.Current, selector);
         while (enrt.MoveNext()) retVal.Check(enrt.Current);
         enrt.Dispose();
         return retVal;
      }
      private static MinMax<T, T2, T3> ExtremumStrict<T, T2, T3>(IEnumerable<T> items, Func<T3, T2> selector)
         where T2 : IComparable
         where T : T3 {
         var retVal = items.Extremum(selector);
         if (!retVal.HasValue) throw new InvalidOperationException("the collection does not contain elements");
         return retVal;
      }
      public static T MaxBy<T, T2, T3>(IEnumerable<T> items, Func<T3, T2> selector)
         where T2 : IComparable
         where T : T3 {
         return ExtremumStrict(items, selector).Max;
      }
      public static T MinBy<T, T2, T3>(IEnumerable<T> items, Func<T3, T2> selector)
         where T2 : IComparable
         where T : T3 {
         return ExtremumStrict(items, selector).Min;
      }
#elif true
      public struct MinMax<T, T2> where T2 : IComparable {
         public T Min, Max;
         public T2 MinVal, MaxVal;
         private Func<T, T2> _Selector;
         public readonly bool HasValue;
         public MinMax(T first, Func<T, T2> selector) {
            _Selector = selector;
            this.Min = first;
            this.Max = first;
            MinVal = _Selector(first);
            MaxVal = MinVal;
            HasValue = true;
         }
         public void Check(T itm) {
            var val = _Selector(itm);
            if (val.CompareTo(MinVal) < 0) {
               Min = itm;
               MinVal = val;
            }
            else if (val.CompareTo(MaxVal) > 0) {
               Max = itm;
               MaxVal = val;
            }
         }
      }
      public static MinMax<T, T2> Extremum<T, T2>(this IEnumerable<T> items, Func<T, T2> selector) where T2 : IComparable {
         var enrt = items.GetEnumerator();
         if (!enrt.MoveNext()) { enrt.Dispose(); return default(MinMax<T, T2>); }
         var retVal = new MinMax<T, T2>(enrt.Current, selector);
         while (enrt.MoveNext()) retVal.Check(enrt.Current);
         enrt.Dispose();
         return retVal;
      }
      private static MinMax<T, T2> ExtremumStrict<T, T2>(IEnumerable<T> items, Func<T, T2> selector) where T2 : IComparable {
         var retVal = items.Extremum(selector);
         if (!retVal.HasValue) throw new InvalidOperationException("the collection does not contain elements");
         return retVal;
      }
      public static T MaxBy<T, T2>(this IEnumerable<T> items, Func<T, T2> selector) where T2 : IComparable {
         return ExtremumStrict(items, selector).Max;
      }
      public static T MinBy<T, T2>(this IEnumerable<T> items, Func<T, T2> selector) where T2 : IComparable {
         return ExtremumStrict(items, selector).Min;
      }
#elif true
#endif
      public static int FindIndex<T>(this IList<T> Subj, Func<T, bool> predicate, int start = 0) {
         for (var i = start; i < Subj.Count; i++) if (predicate(Subj[i])) return i;
         return -1;
      }
      public static int FindIndex<T>(this IList Subj, Func<T, bool> predicate, int start = 0) {
         for (var i = start; i < Subj.Count; i++) if (predicate((T)Subj[i])) return i;
         return -1;
      }
      public static void ForEach<T>(this IEnumerable Subj, Action<T> Action) {
         foreach (T itm in Subj) Action(itm);
      }
      public static void ForEach<T>(this IEnumerable<T> Subj, Action<T> Action) {
         foreach (var itm in Subj) Action(itm);
      }
      public static IEnumerable<T> Except<T>(this IEnumerable<T> Subj, T item) {
         return Subj.Except(new T[] { item });
      }
      public static List<T> ExceptAt<T>(this IEnumerable<T> subj, int index) {
         List<T> retVal = new List<T>();
         using (var enrt = subj.GetEnumerator()) {
            if (index >= 0) {
               for (var i = 0; i++ < index; ) {
                  if (!enrt.MoveNext()) return retVal;
                  retVal.Add(enrt.Current);
               }
               if (!enrt.MoveNext()) return retVal;
            }
            while (enrt.MoveNext()) retVal.Add(enrt.Current);
         }
         return retVal;
      }
      public static List<T2> ToList<T, T2>(this IEnumerable<T> Subj, Func<T, T2> converter) {
         return Subj.Select(converter).ToList();
      }
      public static List<T> ToListFiltered<T>(this IEnumerable<T> Subj, Func<T, bool> filter) {
         return Subj.Where(filter).ToList();
      }

      public static IEnumerable<T> Concat<T>(this IEnumerable<T> Subj, IEnumerable<T> second, IEnumerable<T> third, params IEnumerable<T>[] more) {
         IEnumerable<T>[] src = new IEnumerable<T>[3 + more.Length];
         src[0] = Subj;
         src[1] = second;
         src[2] = third;
         more.CopyX(0, src, 3, more.Length);
         return src.SelectMany(answers => answers);
      }
      #endregion//IEnumerable
   }
}

