﻿using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Globalization;
using System.ComponentModel;
using System.Diagnostics;


namespace System {

   public static class StringX {
      /// <summary> positive value cuts from left, negative cuts from right </summary>
      public static string SubstringX(this string Subj, int cut) {
         if (cut < 0) return Subj.Substring(0, Subj.Length + cut);
         return Subj.Substring(cut);
      }
      /// <summary> start and end instead of start and length. Adds negative numbers to Subj.Length </summary>
      public static string SubstringX(this string Subj, int start, int end) {
         if (start < 0) start += Subj.Length;
         if (end <= 0) end += Subj.Length;
         return Subj.Substring(start, end - start);
      }

      /// <summary> more comfortable than String.Split(separator as String(),count,mode) </summary>
      public static string[] Split(this string Subj, string separator, int count, StringSplitOptions mode = StringSplitOptions.None) {
         return Subj.Split(new string[] { separator }, count, mode);
      }
      public static string Replace(this string Subj, string oldValue, params object[] concatNewValues) {
         return Subj.Replace(oldValue, string.Concat(concatNewValues));
      }

      /// <summary> more comfortable to use than String.Split(separator as String(),mode) </summary>
      public static string[] Split(this string Subj, string separator, StringSplitOptions mode = StringSplitOptions.None) {
         return Subj.Split(new string[] { separator }, mode);
      }

      public static string ReplaceBslashN(this string s) {
         StringBuilder sb = new StringBuilder();
         for (var i = 0; i <= s.Length - 2; i += 2) {
            if (s.IndexOf("\\n", i, 2) < 0) break;
            sb.Append(Environment.NewLine);
         }
         sb.Append(s.Substring(sb.Length));
         return sb.ToString();
      }

      public static string And2(this string Subj, params object[] others) {
         var sb = new StringBuilder(ReplaceBslashN(Subj));
         foreach (var itm in others) {
            sb.Append(ReplaceBslashN(itm.ToString()));
         }
         return sb.ToString();
      }
      ///<summary> verkettet alles</summary>
      public static string And(this string Subj, params object[] others) {
         return string.Concat(Subj, string.Concat(others));
      }

      /// <returns>den Stringabschnitt ab dem letzten Auftreten von sMatch (ohne sMatch)</returns>
      public static string LastRightCut(this string s, string sMatch) {
         var i = s.LastIndexOf(sMatch);
         if (i < 0) return null;
         return s.Substring(i + sMatch.Length);
      }

      /// <returns>den Stringabschnitt ab dem ersten Auftreten von sMatch (ohne sMatch)</returns>
      public static string RightCut(this string S, string sMatch) {
         var I = S.IndexOf(sMatch);
         if (I < 0) return null;
         return S.Substring(I + sMatch.Length);
      }

      public static Regex CreateUnescapeRegex(string Pattern) {
         return new Regex("(?<=(^|[^\\\\](\\\\\\\\)*))" + Pattern);         //der vorangestellte Pattern schließt escaped Matches ('\X' etc.) aus
      }

      /// <summary>gibt den String-Abschnitt links des ersten gefundenen Matches zurück</summary>
      public static string LastLeftCut(this string Subj, string Pattern) {
         var i = Subj.LastIndexOf(Pattern);
         return i >= 0 ? Subj.Remove(i) : null;
      }

      /// <summary>
      /// gibt den String-Abschnitt links des ersten gefundenen Matches zurück
      /// </summary>
      public static string LeftCut(this string Subj, string Pattern) {
         var i = Subj.IndexOf(Pattern);
         return i >= 0 ? Subj.Remove(i) : null;
      }

      public enum Additional { None, Start, End, Start_End }
      public static string Between(this char separator, Additional additional, params object[] args) {
         return Between(args, separator.ToString(), additional);
      }
      public static string Between(this char separator, params object[] args) {
         return Between(args, separator.ToString(), Additional.None);
      }
      public static string Between(this string separator, Additional additional, params object[] args) {
         return Between(args, separator, additional);
      }
      public static string Between(this string separator, params object[] args) {
         return Between(args, separator, Additional.None);
      }

      private static System.Text.StringBuilder _SB = new System.Text.StringBuilder();
      [DebuggerStepThrough()]
      private static string Between(object[] args, string separator, Additional additional) {
         try {            while (args.Length < 2) args = ((IEnumerable)args[0]).Cast<object>().ToArray();         }
         catch (Exception) { }      // can be InvalidCast or IndexOutOfRange
         _SB.Clear();
         if (additional.HasFlag(Additional.Start)) _SB.Append(separator);
         _SB.Append(string.Join(separator, args));
         if (additional.HasFlag(Additional.End)) _SB.Append(separator);
         return _SB.ToString();
      }

      public static int CountChars(this string S, char Search) {
         int functionReturnValue = 0;
         int I = S.IndexOf(Search);
         while (I >= 0) {
            functionReturnValue += 1;
            I = S.IndexOf(Search, I + 1);
         }
         return functionReturnValue;
      }

      public static int CountSubStrings(this string S, string Search, StringComparison Comparison = StringComparison.CurrentCulture) {
         int functionReturnValue = 0;
         int I = S.IndexOf(Search, 0, Comparison);
         while (I >= 0) {
            functionReturnValue += 1;
            I = S.IndexOf(Search, I + 1, Comparison);
         }
         return functionReturnValue;
      }

   }


}
