﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
namespace System.ComponentModel {

   /// <summary>
   /// Usage: derive any MainModel from MainModelBase(Of T) - eg: Public Class MainModel : Inherits MainModelBase(Of MainModel)
   /// </summary>
   public class MainModelBase<T> : NotifyPropertyChanged where T : MainModelBase<T>, new() {

      private static bool _IsProvisionalStatic;
      private static T _Instance;
      public static T Instance {
         get {
            if (_Instance == null) {
               _IsProvisionalStatic = true;
               _Instance = new T();
               _IsProvisionalStatic = false;
            }
            return _Instance;
         }
      }


      public readonly bool IsProvisional;
      public MainModelBase() {
         IsProvisional = _IsProvisionalStatic || NotifyPropertyChanged.IsInDesignMode();
         if (!(IsProvisional || _Instance == null || _Instance.IsProvisional)) {
            var msg = new List<object>();
            msg.Add(string.Join(typeof(T).Name, "You only can instantiate ", " when ", ".Instance.IsNull or ", ".Instance.IsProvisional"));
            //msg.Add(GetType(T).Name.Between( _
            //   "You only can instantiate ", " when ", ".Instance.IsNull or ", ".Instance.IsProvisional"))
            msg.Add("IsProvisional: " + IsProvisional);
            msg.Add("Instance.IsNull: " + _Instance.Null());
            msg.Add("Designmode: " + NotifyPropertyChanged.IsInDesignMode());
            msg.Add("Process: " + Process.GetCurrentProcess().ProcessName);
            msg.Add(Process.GetCurrentProcess().ProcessName);
            if (_Instance != null)
               msg.Add("_Instance.IsProvisional: " + _Instance.IsProvisional);
            throw new Exception(string.Join("\n", msg));
            throw new Exception(string.Format("you only can instantiate {0} when {0}.Instance.IsNull or {0}.Instance.IsProvisional", typeof(T).Name));
         }
         _Instance = (T)this;
      }

   }
}
