﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using System.Windows.Media;
using System.IO;
using System.Windows;
using System.Windows.Data;
using System.Globalization;

namespace PersonList.ViewModel {
   class MainViewModel : MainModelBase<MainViewModel> {

      private char _DataSeparator = Convert.ToChar(0xf018);
      private FileInfo _DataFile = new FileInfo("..\\..\\PersonData.txt");
      private ObservableCollection<Person> __Persons = new ObservableCollection<Person>();
      public ListCollectionView Persons { get; set; }

      private RelayCommand _cmdFill;
      public RelayCommand cmdFill { get { return _cmdFill ?? (_cmdFill = new RelayCommand(Load)); } }
      private RelayCommand _cmdSave;
      public RelayCommand cmdSave { get { return _cmdSave ?? (_cmdSave = new RelayCommand(Save)); } }
      private RelayCommand _cmdDelete;
      public RelayCommand cmdDelete { get { return _cmdDelete ?? (_cmdDelete = new RelayCommand(() => Persons.RemoveAt(Persons.CurrentPosition), () => Persons.CurrentPosition >= 0)); } }
      private RelayCommand _cmdDoSomething;
      public RelayCommand cmdDoSomething { get { return _cmdDoSomething ?? (_cmdDoSomething = new RelayCommand(DoSomethingWithAPerson, () => Persons.CurrentPosition >= 0)); } }
      private RelayCommand _cmdAdd;
      public RelayCommand cmdAdd { get { return _cmdAdd ?? (_cmdAdd = new RelayCommand(() => Persons.AddNewItem(new Person() { Name = "<aName>" }))); } }

      public MainViewModel() {
         Persons = new ListCollectionView(__Persons);
         if (IsProvisional) {  // TestData for Designer-Data-Preview
            Enumerable.Range(0, 10).ForEach(i => __Persons.Add(Person.FromInt(i)));
            Persons.MoveCurrentToFirst();
            return;
         }
         Load();
      }

      private void Save() {
         using (var writer = _DataFile.CreateText()) {
            foreach (var pers in __Persons) {
               writer.WriteLine(_DataSeparator.Between(pers.Name, pers.Phone, pers.BirthDay.ToString(CultureInfo.InvariantCulture), pers.Rate));
            }
         }
         System.Media.SystemSounds.Asterisk.Play();
      }

      private void Load() {
         __Persons.Clear();
         using (var reader = _DataFile.OpenText()) {
            while (!reader.EndOfStream) {
               var data = reader.ReadLine().Split(_DataSeparator);
               __Persons.Add(new Person(data[0], data[1], System.DateTime.Parse(data[2], CultureInfo.InvariantCulture), int.Parse(data[3])));
            }
         }
         Persons.MoveCurrentToFirst();
      }

      private void DoSomethingWithAPerson() {
         var person = __Persons[Persons.CurrentPosition];         //  access the currently selected Person
         MessageBox.Show("Aktuell angewählte Person: " + person.Name);
      }

      private void __Persons_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e) {
         if (e.Action != System.Collections.Specialized.NotifyCollectionChangedAction.Remove) return;
         if (!Persons.MoveCurrentToNext()) Persons.MoveCurrentToPrevious();   // ensure always a person is selected (as long as any exists)
      }

   }
}
